@@ -9,7 +9,6 @@ from xml.dom import minidom
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element, SubElement
 from sanitize_filename import sanitize
-
 import boto3
 from boto3.s3.transfer import TransferConfig
 from botocore.config import Config
@@ -311,7 +310,7 @@ def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Tit
 
     if additional_namespaces is not None:
         for prefix, uri in additional_namespaces.items():
-            xml.etree.ElementTree.SubElement(xml_index, "namespaceMapping",  {"key": prefix, "value": uri})
+            xml.etree.ElementTree.SubElement(xml_index, "namespaceMapping", {"key": prefix, "value": uri})
 
     xml_request = xml.etree.ElementTree.tostring(xml_index, encoding='utf-8', xml_declaration=True)
     search_xml = root_element + "-index.xml"
@@ -589,6 +588,64 @@ def simple_asset_package(preservation_file=None, access_file=None, export_folder
 
 class UploadAPI(AuthenticatedAPI):
 
+    def ingest_web_video(self, url=None, parent_folder=None, **kwargs):
+        try:
+            import youtube_dl
+        except ImportError:
+            raise RuntimeError("Package youtube_dl is required for this method. pip install --upgrade youtube-dl")
+
+        ydl_opts = {}
+
+        def my_hook(d):
+            if d['status'] == 'finished':
+                print('Download Complete. Uploading to Preservica ...')
+
+        ydl_opts = {
+            'outtmpl': '%(id)s.mp4',
+            'progress_hooks': [my_hook],
+        }
+
+        with youtube_dl.YoutubeDL(ydl_opts) as ydl:
+            meta = ydl.extract_info(url, download=True)
+
+            vid_id = meta.get('id')
+
+            if 'Title' in kwargs:
+                title = kwargs.get("Title")
+            else:
+                title = meta.get('title')
+
+            if 'Description' in kwargs:
+                description = kwargs.get("Description")
+            else:
+                description = meta.get('description')
+
+            if 'SecurityTag' in kwargs:
+                security_tag = kwargs.get("SecurityTag")
+            else:
+                security_tag = "open"
+
+            if 'Identifiers' in kwargs:
+                identifier_map = kwargs.get('Identifiers')
+                identifier_map["Video-ID"] = vid_id
+            else:
+                identifier_map = {"Video-ID": vid_id}
+
+            if 'Asset_Metadata' in kwargs:
+                descriptive_metadata = kwargs.get('Asset_Metadata')
+            else:
+                descriptive_metadata = {}
+
+            upload_date = meta.get('upload_date')
+            duration = meta.get('duration')
+
+            package = simple_asset_package(preservation_file=f"{vid_id}.mp4", parent_folder=parent_folder, Title=title,
+                                           Description=description, Identifiers=identifier_map,
+                                           Asset_Metadata=descriptive_metadata,
+                                           Preservation_Content_Title=title, SecurityTag=security_tag)
+
+            self.upload_zip_package(path_to_zip_package=package, folder=parent_folder)
+
     def upload_zip_package(self, path_to_zip_package, folder=None, callback=None, delete_after_upload=False):
         bucket = f'{self.tenant.lower()}.package.upload'
         endpoint = f'https://{self.server}/api/s3/buckets'
